@@ -32,7 +32,15 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include "glib.h"
+
+#include "garray.h"
+
+#include "gmem.h"
+#include "gthread.h"
+#include "gmessages.h"
+#include "gqsort.h"
+
+#include "galias.h"
 
 
 #define MIN_ARRAY_SIZE  16
@@ -639,6 +647,29 @@ g_ptr_array_sort_with_data (GPtrArray        *array,
 		     user_data);
 }
 
+/**
+ * g_ptr_array_foreach:
+ * @array: a #GPtrArray
+ * @func: the function to call for each array element
+ * @user_data: user data to pass to the function
+ * 
+ * Calls a function for each element of a #GPtrArray.
+ *
+ * Since: 2.4
+ **/
+void
+g_ptr_array_foreach (GPtrArray *array,
+                     GFunc      func,
+                     gpointer   user_data)
+{
+  guint i;
+
+  g_return_if_fail (array);
+
+  for (i = 0; i < array->len; i++)
+    (*func) (array->pdata[i], user_data);
+}
+
 /* Byte arrays 
  */
 
@@ -705,9 +736,9 @@ g_byte_array_remove_range (GByteArray *array,
                            guint index_,
                            guint length)
 {
-  g_return_val_if_fail (array, FALSE);
-  g_return_val_if_fail (index_ < array->len, FALSE);
-  g_return_val_if_fail (index_ + length <= array->len, FALSE);
+  g_return_val_if_fail (array, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
+  g_return_val_if_fail (index_ + length <= array->len, NULL);
 
   return (GByteArray *)g_array_remove_range ((GArray*) array, index_, length);
 }
@@ -726,3 +757,6 @@ g_byte_array_sort_with_data (GByteArray       *array,
 {
   g_array_sort_with_data ((GArray *) array, compare_func, user_data);
 }
+
+#define __G_ARRAY_C__
+#include "galiasdef.c"
