@@ -587,8 +587,11 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
 
   found_file = FALSE;
   data_dirs = all_data_dirs;
+  output_path = NULL;
   while (*data_dirs != NULL && !found_file)
     {
+      g_free (output_path);
+
       fd = find_file_in_data_dirs (file, &output_path, &data_dirs, 
                                    &key_file_error);
       
@@ -596,7 +599,7 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
         {
           if (key_file_error)
             g_propagate_error (error, key_file_error);
-	  break;
+ 	  break;
         }
 
       found_file = g_key_file_load_from_fd (key_file, fd, flags,
@@ -606,15 +609,17 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
       if (key_file_error)
         {
 	  g_propagate_error (error, key_file_error);
-          g_free (output_path);
 	  break;
         }
-      
-      if (full_path)
-	*full_path = output_path;
     }
 
+  if (found_file && full_path)
+    *full_path = output_path;
+  else 
+    g_free (output_path);
+
   g_strfreev (all_data_dirs);
+
   return found_file;
 }
 
@@ -1131,7 +1136,7 @@ g_key_file_get_groups (GKeyFile *key_file,
  * and @error is set to #G_KEY_FILE_ERROR_GROUP_NOT_FOUND.
  *
  * Return value: a newly allocated string or %NULL if the specified 
- * key cannot be found.
+ *  key cannot be found.
  *
  * Since: 2.6
  **/
@@ -1237,7 +1242,7 @@ g_key_file_set_value (GKeyFile    *key_file,
  * and @error is set to #G_KEY_FILE_ERROR_GROUP_NOT_FOUND.
  *
  * Return value: a newly allocated string or %NULL if the specified 
- * key cannot be found.
+ *   key cannot be found.
  *
  * Since: 2.6
  **/
@@ -1518,9 +1523,9 @@ extern GSList *_g_compute_locale_variants (const gchar *locale);
  * with @key cannot be interpreted or no suitable translation can
  * be found then the untranslated value is returned.
  *
- * Return value: a newly allocated string or %NULL if the specified key 
- *  cannot be found.
- * 
+ * Return value: a newly allocated string or %NULL if the specified 
+ *   key cannot be found.
+ *
  * Since: 2.6
  **/
 gchar *
@@ -1608,7 +1613,7 @@ g_key_file_get_locale_string (GKeyFile     *key_file,
  *
  * Returns the values associated with @key under @group_name
  * translated in the given @locale if available.  If @locale is
- * %NULL then the current locale is assumed. 
+ * %NULL then the current locale is assumed.
 
  * If @key cannot be found then %NULL is returned and @error is set to
  * #G_KEY_FILE_ERROR_KEY_NOT_FOUND. If the values associated
@@ -2003,7 +2008,7 @@ g_key_file_get_integer (GKeyFile     *key_file,
  * @value: an integer value
  *
  * Associates a new integer value with @key under @group_name.
- * If @key cannot be found then it is created. 
+ * If @key cannot be found then it is created.
  *
  * Since: 2.6
  **/
@@ -2033,7 +2038,7 @@ g_key_file_set_integer (GKeyFile    *key_file,
  * @error: return location for a #GError
  *
  * Returns the values associated with @key under @group_name as
- * integers. If @group_name is %NULL, the start group is used.
+ * integers. 
  *
  * If @key cannot be found then the return value is undefined and
  * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
@@ -3167,15 +3172,30 @@ g_key_file_parse_value_as_integer (GKeyFile     *key_file,
 				   GError      **error)
 {
   gchar *end_of_valid_int;
-  gint int_value = 0;
+  glong long_value;
+  gint int_value;
 
-  int_value = strtol (value, &end_of_valid_int, 10);
+  errno = 0;
+  long_value = strtol (value, &end_of_valid_int, 10);
 
-  if (*end_of_valid_int != '\0')
-    g_set_error (error, G_KEY_FILE_ERROR,
-		 G_KEY_FILE_ERROR_INVALID_VALUE,
-		 _("Value '%s' cannot be interpreted as a number."), value);
+  if (*value == '\0' || *end_of_valid_int != '\0')
+    {
+      g_set_error (error, G_KEY_FILE_ERROR,
+		   G_KEY_FILE_ERROR_INVALID_VALUE,
+		   _("Value '%s' cannot be interpreted as a number."), value);
+      return 0;
+    }
 
+  int_value = long_value;
+  if (int_value != long_value || errno == ERANGE)
+    {
+      g_set_error (error,
+		   G_KEY_FILE_ERROR, 
+		   G_KEY_FILE_ERROR_INVALID_VALUE,
+		   _("Integer value '%s' out of range"), value);
+      return 0;
+    }
+  
   return int_value;
 }
 
